package cn.palmte.work.controller.backend;


import cn.palmte.work.bean.ResponseMsg;
import cn.palmte.work.model.ActScript;
import cn.palmte.work.model.ActScriptRepository;
import cn.palmte.work.service.ActScriptService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 流程脚本管理
 */
@Controller
@RequestMapping("/actScript")
public class ActScriptController extends BaseController {

    private static final Logger logger = LoggerFactory.getLogger(ActScriptController.class);

    @Autowired
    private ActScriptService actScriptService;

    @Autowired
    private ActScriptRepository actScriptRepository;

    @RequestMapping("/list")
    public String list(@RequestParam(value = "keywords", required = false) String keywords,
                       @RequestParam(value = PAGE_NUMBER, defaultValue = DEFAULT_PAGE_NUMBER) int pageNumber,
                       @RequestParam(value = PAGE_SIZE, defaultValue = DEFAULT_PAGE_SIZE) int pageSize,
                       Map<String, Object> model) {
        ConcurrentHashMap<String, String> searchInfo = getSearchInfo(keywords, model);
        model.put("pager", actScriptService.list(searchInfo, pageNumber, pageSize));
        return "/admin/act_script_list";
    }

    @GetMapping(value = "/add")
    public String add(Map<String, Object> model) {
        List<String> list = getScriptList();
        model.put("actScript", new ActScript());
        model.put("classList", list);

        List<String> methodList = new ArrayList<>();
        for (String l : list) {
            methodList.addAll(getMethodList(l));
        }

        model.put("methodList", methodList);
        return "/admin/act_script_input";
    }

    private List<String> getScriptList() {
        List<String> list = new ArrayList<>(1);
        list.add("cn.palmte.work.service.ActCallbackScript");
        return list;
    }

    private List<String> getMethodList(String className) {
        List<String> list = new ArrayList<>();

        try {
            Method[] methods = Class.forName(className).getDeclaredMethods();
            for (Method method : methods) {
                list.add(method.getName());
            }
        } catch (ClassNotFoundException e) {
            logger.error("", e);
        }

        return list;
    }

    @GetMapping(value = "/edit")
    public String edit(@RequestParam int id, Map<String, Object> model) {
        List<String> list = getScriptList();
        model.put("actScript", actScriptRepository.findOne(id));
        model.put("classList", list);
        List<String> methodList = new ArrayList<>();
        for (String l : list) {
            methodList.addAll(getMethodList(l));
        }

        model.put("methodList", methodList);
        return "/admin/act_script_input";
    }

    @RequestMapping("/save")
    public String save(ActScript actScript) {
        actScriptService.save(actScript);
        return "redirect:/actScript/list";
    }


    @ResponseBody
    @GetMapping(value = "/delete")
    public ResponseMsg delete(@RequestParam("ids") String ids){
        if ("".equals(ids)){
            return ResponseMsg.buildFailedMsg("删除失败，无选中项！");
        }else {
            String[] deleteIds=ids.split("#%#");
            for (String id : deleteIds) {
                actScriptService.delete(Integer.parseInt(id));
            }
           return ResponseMsg.buildSuccessMsg("删除成功");
        }
    }


}
